//+------------------------------------------------------------------+
//|                                                    T3 DPO v2.mq4 |
//+------------------------------------------------------------------+

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 LimeGreen
#property indicator_color2 Magenta
#property indicator_color3 DeepSkyBlue
#property indicator_color4 Red
#property indicator_width2 2
#property indicator_width3 2
#property indicator_width4 2

//
//
//
//
//

extern string TimeFrame   = "Current time frame";
extern int    T3Period    = 8;
extern double T3Hot       = 0.7;
extern bool   T3Original  = false;
extern int    UsePrice    = PRICE_CLOSE;
extern bool   Interpolate = true;

//
//
//
//
//

extern string _                     = "arrows settings";
extern bool   showArrowsOnChart     = true;
extern bool   showArrowsOnIndicator = true;
extern string arrowsIdentifier      = "T3_dpo_arrows";
extern color  arrowsUpColor         = DeepSkyBlue;
extern color  arrowsDnColor         = Red;

extern string __                    = "alerts settings";
extern bool   alertsOn              = true;
extern bool   alertsOnCurrent       = true;
extern bool   alertsMessage         = true;
extern bool   alertsSound           = false;
extern bool   alertsEmail           = false;

//
//
//
//
//

double t3[];
double priceLine[];
double arrowUp[];
double arrowDn[];
double trend[];

//
//
//
//
//

int    timeFrame;
string IndicatorFileName;
bool   ReturningBars;
bool   CalculatingDpo;

double alpha;
double c1;
double c2;
double c3;
double c4;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,t3);       SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,priceLine);
   SetIndexBuffer(2,arrowUp);
   SetIndexBuffer(3,arrowDn);
   SetIndexBuffer(4,trend);
      if (showArrowsOnIndicator)
      {
         SetIndexStyle(2,DRAW_ARROW); SetIndexArrow(2,159);
         SetIndexStyle(3,DRAW_ARROW); SetIndexArrow(3,159);
      }
      else
      {
         SetIndexStyle(2,DRAW_NONE);
         SetIndexStyle(3,DRAW_NONE);
      }

   //
   //
   //
   //
   //
   
      double a  = T3Hot;
             c1 = -a*a*a;
             c2 =  3*(a*a+a*a*a);
             c3 = -3*(2*a*a+a+a*a*a);
             c4 = 1+3*a+a*a*a+3*a*a;

      T3Period = MathMax(1,T3Period);
      if (T3Original)
           alpha = 2.0/(1.0 + T3Period);
      else alpha = 2.0/(2.0 + (T3Period-1.0)/2.0);

      //
      //
      //
      //
      //
         
      IndicatorFileName = WindowExpertName();
      CalculatingDpo    = (TimeFrame=="CalculateDpo"); if (CalculatingDpo) return(0);
      ReturningBars     = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
      
   //
   //
   //
   //
   //
         
   IndicatorShortName("T3 dpo "+timeFrameToString(timeFrame)+" ("+T3Period+","+DoubleToStr(T3Hot,2)+")");
   return(0);
}
int deinit()
{
   if (showArrowsOnChart) deleteArrows();
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (ReturningBars) { t3[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (CalculatingDpo || timeFrame==Period())
   {
      for(i=limit; i>=0; i--)
      {
         t3[i]        = iT3(iMA(NULL,0,T3Period,0,MODE_SMA,UsePrice,i),i,0);
         priceLine[i] =     iMA(NULL,0,1       ,0,MODE_SMA,UsePrice,i);
         
         //
         //
         //
         //
         //
         
         if (!CalculatingDpo) manageArrows(i);
      }
      if (!CalculatingDpo) manageAlerts();
      return(0);
   }
   
   //
   //
   //
   //
   //
         
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         t3[i]        = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateDpo",T3Period,T3Hot,T3Original,UsePrice,0,y);
         priceLine[i] = iMA(NULL,timeFrame,1,0,MODE_SMA,UsePrice,y);
            manageArrows(i);
         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++)
            {
               t3[i+k]        = k*factor*t3[i+n]        + (1.0-k*factor)*t3[i];
               priceLine[i+k] = k*factor*priceLine[i+n] + (1.0-k*factor)*priceLine[i];
            }               
   }
   manageAlerts();
      
   //
   //
   //
   //
   //
   
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double _t3Values[][6];
double iT3(double price,int i,int s)
{
   if (ArrayRange(_t3Values,0) != Bars) ArrayResize(_t3Values,Bars);

   int r = Bars-i-1;
   if (r < 2)
      {
         _t3Values[r][s+0] = price;
         _t3Values[r][s+1] = price;
         _t3Values[r][s+2] = price;
         _t3Values[r][s+3] = price;
         _t3Values[r][s+4] = price;
         _t3Values[r][s+5] = price;
      }
   else
      {
         _t3Values[r][s+0] = _t3Values[r-1][s+0]+alpha*(price            -_t3Values[r-1][s+0]);
         _t3Values[r][s+1] = _t3Values[r-1][s+1]+alpha*(_t3Values[r][s+0]-_t3Values[r-1][s+1]);
         _t3Values[r][s+2] = _t3Values[r-1][s+2]+alpha*(_t3Values[r][s+1]-_t3Values[r-1][s+2]);
         _t3Values[r][s+3] = _t3Values[r-1][s+3]+alpha*(_t3Values[r][s+2]-_t3Values[r-1][s+3]);
         _t3Values[r][s+4] = _t3Values[r-1][s+4]+alpha*(_t3Values[r][s+3]-_t3Values[r-1][s+4]);
         _t3Values[r][s+5] = _t3Values[r-1][s+5]+alpha*(_t3Values[r][s+4]-_t3Values[r-1][s+5]);
      }
   return(c1*_t3Values[r][s+5] + c2*_t3Values[r][s+4] + c3*_t3Values[r][s+3] + c4*_t3Values[r][s+2]);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//
   
void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"up");
         if (trend[whichBar] ==-1) doAlert(whichBar,"down");
      }         
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol(),timeFrameToString(timeFrame)+" at ",TimeToStr(TimeLocal(),TIME_SECONDS)," T3 DPO trend changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"T3 DPO trend "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrows(int i)
{
   trend[i] = trend[i+1];
      if (priceLine[i]>t3[i]) trend[i] =  1;
      if (priceLine[i]<t3[i]) trend[i] = -1;
      if (showArrowsOnChart)
      {
         deleteArrow(Time[i]);
         if (trend[i]!=trend[i+1])
         {
            if (trend[i] == 1) drawArrow(i,arrowsUpColor,241,false);
            if (trend[i] ==-1) drawArrow(i,arrowsDnColor,242,true);
          }
      }
      if (showArrowsOnIndicator)
      {
         arrowUp[i] = EMPTY_VALUE;
         arrowDn[i] = EMPTY_VALUE;
         if (trend[i]!=trend[i+1])
         {
            if (trend[i] == 1) arrowUp[i] = t3[i];
            if (trend[i] ==-1) arrowDn[i] = t3[i];
         }
      }
}               

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}
void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
string StringUpperCase(string str) { StringToUpper(str); return(str); }